from pelican import signals

import multiprocessing
import htmlmin
import os
import re

import logging
logger = logging.getLogger(__name__)


def run(pelican):

    # should we be minifying HTML files
    if pelican.settings.get(
            'HTMLMIN_ENABLED',
            not logger.getEffectiveLevel() <= logging.DEBUG
    ) is False:
        logger.debug("'pelican-htmlmin' disabled")
        return

    options = pelican.settings.get(
        'HTMLMIN_OPTIONS',
        {
            'remove_comments': True,
            'remove_all_empty_space': True,
            'remove_optional_attribute_quotes': False
        }
    )
    htmlfile = re.compile(
        pelican.settings.get('HTMLMIN_MATCH', r'.html?$')
    )
    pool = multiprocessing.Pool()

    # find all matching files and give to workers to minify
    for base, dirs, files in os.walk(pelican.settings['OUTPUT_PATH']):
        for f in filter(htmlfile.search, files):
            filepath = os.path.join(base, f)
            pool.apply_async(worker, (filepath, options))

    # wait for the workers to finish
    pool.close()
    pool.join()


def worker(filepath, options):
    """use htmlmin to minify the given file"""

    # copy file into memory while we minify it
    with open(filepath, encoding='utf-8') as f:
        rawhtml = f.read()

    # minify 'rawhtml' and save the minified
    # version back to the same location
    with open(filepath, 'w', encoding='utf-8') as f:
        logger.debug('Minifying: %s', filepath)
        try:
            compressed = htmlmin.minify(rawhtml, **options)
            f.write(compressed)
        except Exception as e:
            logger.critical('Minification failed: %s', e)


def register():
    """minify HTML at the end of the pelican build"""
    signals.finalized.connect(run)



from setuptools import setup
from io import open
import os

pwd = os.path.abspath(os.path.dirname(__file__))

VERSION = os.environ.get('CI_COMMIT_TAG', '0.0.0')

REQUIRES = [
    'htmlmin',
]

description = open(os.path.join(pwd, 'readme.rst'), encoding='utf-8').read()

setup(
    name='pelican-htmlmin',
    packages=['pelican_htmlmin'],

    version=VERSION,
    url='https://gitlab.com/bryanbrattlof/pelican-htmlmin',

    description="Minifies HTML files generated by Pelican",
    long_description=description,

    author='Bryan Brattlof',
    author_email='hello@bryanbrattlof.com',

    license='MIT',

    install_requires=REQUIRES,

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',

        'Framework :: Pelican',
        'Framework :: Pelican :: Plugins',

        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',

        'Natural Language :: English',
    ],
)
